@@ -1,7 +1,7 @@
 /*
  * gui-bar-window.c - bar window functions (used by all GUI)
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,6 +30,7 @@
 
 #include "../core/weechat.h"
 #include "../core/wee-config.h"
+#include "../core/wee-hashtable.h"
 #include "../core/wee-hdata.h"
 #include "../core/wee-infolist.h"
 #include "../core/wee-log.h"
@@ -827,7 +828,7 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
             total_items = 0;
             max_length = 1;
             max_length_screen = 1;
-            split_items = malloc(bar_window->items_count * sizeof(*split_items));
+            split_items = malloc (bar_window->items_count * sizeof (*split_items));
             for (i = 0; i < bar_window->items_count; i++)
             {
                 if (bar_window->items_subcount[i] > 0)
@@ -839,8 +840,15 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
                                                                   i, sub);
                         if (ptr_content && ptr_content[0])
                         {
-                            split_items[i][sub] = string_split (ptr_content,
-                                                                "\n", 0, 0, NULL);
+                            split_items[i][sub] = string_split (
+                                ptr_content,
+                                "\n",
+                                NULL,
+                                WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                0,
+                                NULL);
                             for (j = 0; split_items[i][sub][j]; j++)
                             {
                                 total_items++;
@@ -915,6 +923,7 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
                     if (content)
                         free (content);
                     free (split_items);
+                    free (linear_items);
                     return NULL;
                 }
                 content = content2;
@@ -1520,19 +1529,60 @@ gui_bar_window_scroll (struct t_gui_bar_window *bar_window,
 }
 
 /*
- * Returns hdata for bar window.
+ * Callback for updating data of a bar window.
  */
 
-struct t_hdata *
-gui_bar_window_hdata_bar_window_cb (void *data, const char *hdata_name)
+int
+gui_bar_window_update_cb (void *data, struct t_hdata *hdata, void *pointer,
+                          struct t_hashtable *hashtable)
 {
-    struct t_hdata *hdata;
+    const char *value;
+    int rc;
 
     /* make C compiler happy */
     (void) data;
 
+    rc = 0;
+
+    if (hashtable_has_key (hashtable, "scroll_x"))
+    {
+        value = hashtable_get (hashtable, "scroll_x");
+        if (value)
+        {
+            hdata_set (hdata, pointer, "scroll_x", value);
+            rc++;
+        }
+    }
+
+    if (hashtable_has_key (hashtable, "scroll_y"))
+    {
+        value = hashtable_get (hashtable, "scroll_y");
+        if (value)
+        {
+            hdata_set (hdata, pointer, "scroll_y", value);
+            rc++;
+        }
+    }
+
+    return rc;
+}
+
+/*
+ * Returns hdata for bar window.
+ */
+
+struct t_hdata *
+gui_bar_window_hdata_bar_window_cb (const void *pointer, void *data,
+                                    const char *hdata_name)
+{
+    struct t_hdata *hdata;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+
     hdata = hdata_new (NULL, hdata_name, "prev_bar_window", "next_bar_window",
-                       0, 0, NULL, NULL);
+                       0, 0, &gui_bar_window_update_cb, NULL);
     if (hdata)
     {
         HDATA_VAR(struct t_gui_bar_window, bar, POINTER, 0, NULL, "bar");
@@ -1540,8 +1590,8 @@ gui_bar_window_hdata_bar_window_cb (void *data, const char *hdata_name)
         HDATA_VAR(struct t_gui_bar_window, y, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, width, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, height, INTEGER, 0, NULL, NULL);
-        HDATA_VAR(struct t_gui_bar_window, scroll_x, INTEGER, 0, NULL, NULL);
-        HDATA_VAR(struct t_gui_bar_window, scroll_y, INTEGER, 0, NULL, NULL);
+        HDATA_VAR(struct t_gui_bar_window, scroll_x, INTEGER, 1, NULL, NULL);
+        HDATA_VAR(struct t_gui_bar_window, scroll_y, INTEGER, 1, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, cursor_x, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, cursor_y, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_bar_window, current_size, INTEGER, 0, NULL, NULL);
